// adp2asc.c : This file contains the 'main' function. Program execution begins and ends there.
//
// $Date: 2019-02-04 13:46:28 +0100 (ma, 04 feb 2019) $
// $Revision: 24 $
// $Author: jhuizinga $
// $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/bin2asc/sourceADV/adv2asc.c $
//

//#include "pch.h"
#include "aqd2asc.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
  FILE            *infile;
  FILE            *outfile;
  FILE            *logfile;

  errno_t         err;

  unsigned char   cSync;
  unsigned char   cId;
  PdHrProf        pdHrProf;
  int             i, j;
  unsigned short  lChecksum;
  unsigned short  nChecksum = 0;
  char            *bError;
  char            *bStatus;
  size_t          nElements;
  long            positie = 0;
  long            size = 0;
  if (argc != 4)
  {
    printf("Usage: aqd2asc infile outfile logfile");
    exit(1);
  }

  // Open file's
  err = fopen_s(&logfile, argv[3], "a");
  if (err != 0) {
    fprintf(logfile, "Error : opening logfile file %s\n", argv[3]);
    exit(1);
  }
  err = fopen_s(&infile, argv[1], "rb");
  if (err != 0) {
    fprintf(logfile, "Error : opening input file %s\n", argv[1]);
    exit(1);
  }
  err = fopen_s(&outfile, argv[2], "w");
  if (err != 0) {
    fprintf(logfile, "Error : opening output file %s\n", argv[2]);
    exit(1);
  }


  fprintf(logfile, "Conversie naar ascii van %s naar %s gestart!\n", argv[1], argv[2]);

  // Eenmalig weergeven wat filenaam is
  fprintf(outfile, "Filename: %s \n", argv[1]);
  // Weergeven welk conversie programma
  fprintf(outfile, "Conversie naar ascii: %s \n", "AQD2ASC_$Revision: 24 $");

  // Aanmaken van de header voor de meta-info
  fprintf(outfile,
    "sync,"
    "id,"
    "structsize,"
    "year,"
    "month,"
    "day,"
    "hours,"
    "minutes,"
    "seconds,"
    "milliseconds,"
    "errorcode,"
    "battery_voltage(0.1V),"
    "soundspeed(0.1m/s),"
    "hHeading(0.1deg),"
    "hPitch(0.1deg),"
    "hRoll(0.1deg),"
    "cPressureMSB (mm),"
    "hStatus,"
    "hPressureLSW (mm),"
    "hTemperature (0.01 deg Celsius),"
    "hAnaIn1,"
    "hAnaIn2,"
    "nBeams,"
    "nCells,"
    "hVelLag2[0],"
    "hVelLag2[1],"
    "hVelLag2[2],"
    "hAmpLag2[0],"
    "hAmpLag2[1],"
    "hAmpLag2[2],"
    "hCorrLag2[0],"
    "hCorrLag2[1],"
    "hCorrLag2[2],"
    "hSpare1,"
    "hSpare2,"
    "hSpare3\n");

  fseek(infile, 0L, SEEK_END);
  size = ftell(infile);
  fseek(infile, 0L, SEEK_SET);

  while (!feof(infile)) {

    fread(&cSync, sizeof(cSync), 1, infile);
    fread(&cId, sizeof(cId), 1, infile);

    if (cSync == 165 && cId == 42) {
      positie = (ftell(infile) - 2L);
      fseek(infile, positie, 0);

      nElements = fread(&pdHrProf, sizeof(pdHrProf), 1, infile);
      if (nElements  > 0) {

        if (pdHrProf.nBeams != 3) {
          // Handle the error.
          fprintf(logfile, "Number of beams not 3 \n");
          exit(1);
        }
        if (pdHrProf.nCells != 15) {
          // Handle the error.
          fprintf(logfile, "Number of cells not 15 \n");
          exit(1);
        }

        lChecksum = Checksum((short *)&pdHrProf, (sizeof(pdHrProf) / 2) - 1);
        if (pdHrProf.hChecksum != lChecksum) {
          // Handle the error.
          nChecksum++;
          fprintf(logfile, "Checksum error %d != %d\n", pdHrProf.hChecksum, lChecksum);
        }
        else {

          bError = decimal_to_binary(pdHrProf.hError);
          bStatus = decimal_to_binary(pdHrProf.cStatus);

          fprintf(outfile, "%x,%x,%d,%x,%x,%x,%x,%x,%x,%d,%s,%d,%d,%d,%d,%d,%x,%s,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
            pdHrProf.cSync,
            pdHrProf.cId,
            pdHrProf.hSize,
            pdHrProf.clock.cYear,
            pdHrProf.clock.cMonth,
            pdHrProf.clock.cDay,
            pdHrProf.clock.cHour,
            pdHrProf.clock.cMinute,
            pdHrProf.clock.cSecond,
            pdHrProf.hMilliseconds,
            bError,
            pdHrProf.hBattery,
            pdHrProf.hSoundSpeed,
            pdHrProf.hHeading,
            pdHrProf.hPitch,
            pdHrProf.hRoll,
            pdHrProf.cPressureMSB,
            bStatus,
            pdHrProf.hPressureLSW,
            pdHrProf.hTemperature,
            pdHrProf.hAnaIn1,
            pdHrProf.hAnaIn2,
            pdHrProf.nBeams,
            pdHrProf.nCells,
            pdHrProf.hVelLag2[0],
            pdHrProf.hVelLag2[1],
            pdHrProf.hVelLag2[2],
            pdHrProf.hAmpLag2[0],
            pdHrProf.hAmpLag2[1],
            pdHrProf.hAmpLag2[2],
            pdHrProf.hCorrLag2[0],
            pdHrProf.hCorrLag2[1],
            pdHrProf.hCorrLag2[2]);

          fprintf(outfile, "hVel\n");
          for (i = 0; i < pdHrProf.nBeams; i++) {
            for (j = 0; j < pdHrProf.nCells; j++) {
              fprintf(outfile, "%d,", pdHrProf.hVel[i][j]);
            }
            fprintf(outfile, "\n");
          }

          fprintf(outfile, "cAmp\n");
          for (i = 0; i < pdHrProf.nBeams; i++) {
            for (j = 0; j < pdHrProf.nCells; j++) {
              fprintf(outfile, "%d,", pdHrProf.cAmp[i][j]);
            }
            fprintf(outfile, "\n");
          }

          fprintf(outfile, "cCorr\n");
          for (i = 0; i < pdHrProf.nBeams; i++) {
            for (j = 0; j < pdHrProf.nCells; j++) {
              fprintf(outfile, "%d,", pdHrProf.cCorr[i][j]);
            }
            fprintf(outfile, "\n");
          }
        }
      }
    }
  }

  fclose(infile);
  fclose(outfile);
  fprintf(logfile, "Number of checksum errors = %d\n", nChecksum);
  fprintf(logfile, "Conversie naar ascii van %s naar %s einde!\n", argv[1], argv[2]);
  fclose(logfile);

  return 0;
}

char *decimal_to_binary(char n)
{
  int c, d, count;
  char *pointer;

  count = 0;
  pointer = (char*)malloc(8 + 1);

  if (pointer == NULL)
    exit(EXIT_FAILURE);

  for (c = 7; c >= 0; c--)
  {
    d = n >> c;

    if (d & 1)
      *(pointer + count) = 1 + '0';
    else
      *(pointer + count) = 0 + '0';

    count++;
  }
  *(pointer + count) = '\0';

  return  pointer;
}

unsigned short Checksum(short *phBuff, int n) {
  int i;

  unsigned short hChecksum = 0xb58c;

  for (i = 0; i < n; i++) {
    hChecksum += phBuff[i];
  }
  return hChecksum;
}
